/*! \file structures.h
 * \brief Fichier contenant les principales structures.
 * Definition des structures necessaires :
 * SOMMAIRE
 * Type_terrain
 * Case
 * Attaque
 * Classe
 * Personnage
 * Effet
 * Competence
 * Joueur
 * NodeJoueur
 * ListJoueur
 * Partie
 */

#ifndef STRUCTURES_H_INCLUDED
#define STRUCTURES_H_INCLUDED

/*! \def TAILLE_MAX_CARTE */
#define TAILLE_MAX_CARTE 256
/*! \def TAILLE_ID */
#define TAILLE_ID 8
/*! \def TAILLE_NOMS */
#define TAILLE_NOMS 16
/*! \def TAILLE_MAX_GROUPE */
#define TAILLE_MAX_GROUPE 3
/*! \def LARG_MAX_CARTE */
#define LARG_MAX_CARTE 16
/*! \def LONG_MAX_CARTE */
#define HAUT_MAX_CARTE 16

/*! \enum boolean
 * \brief Definition du type Boolean.
 *
 * faux=0, vrai=1
 */
typedef enum {faux, vrai} boolean;

/*! \struct Skins_Charac structures.h
 * \brief Definition de la structure Skins_Charac
 *
 * Ensemble de noms de fichiers indiquant les textures des personnages
 */
 typedef struct Skins_Charac{
    char charac_default[50];
    char charac_selected[50];
    char charac_healed[50];
    char charac_damaged[50];
} Skins_Charac;


/*! \struct Type_terrain structures.h
 * \brief Definition de la structure Type_terrain
 */
typedef struct Terrain{
    char nomTerrain[TAILLE_NOMS];/*!< Nom du terrain. */
    boolean franchissable;/*!< Le terrain est franchissable ou non par un personnage. */
    boolean franchissable_sorts;/*!< Le terrain est franchissable ou non par un sort. */
    unsigned short int PD_requis;/*!< Nombre de PD requis pour traverser le terrain. */
}Terrain;

/*! \struct Case structures.h
 * \brief Definition de la structure Case
 */
typedef struct Case{
    unsigned short int coord_x; /*!< Abscisse de la case*/
    unsigned short int coord_y; /*!< Ordonnee de la case*/
    Terrain * terrain; /*!< Le type de terrain de la case*/
    boolean occupee; /*!< La case est occupee ou non par un personnage*/
} Case;

/*! \typedef carte structures.h
 * \brief Definition du type Carte
 *
 * La carte est un tableau de Case de taille TAILLE_MAX_CARTE.
 */
typedef Case Carte[TAILLE_MAX_CARTE];

/*! \struct Effet structures.h
 * \brief Definition de la structure Effet.
 */
typedef struct Effet{
    char id[TAILLE_ID]; /*!< ID de l'effet de taille TAILLE_ID*/
    char nom[TAILLE_NOMS]; /*!< Nom de l'effet de taille TAILLE_NOMS*/
    unsigned int nb_tours;/*!< Le nombre de tours restants avant que l'effet ne s'estompe*/
    unsigned int valeur_degats; /*!< Le nombre de degat subit */
    unsigned int valeur_soin; /*!< Le nombre PV regeneres */
    unsigned int valeur_degats_duree; /*!< Le nombre de degat de duree subit par tour*/
    unsigned int valeur_soin_duree; /*!< Le nombre PV regeneres par tour*/
    unsigned int paralysie; /*!< L'effet provoque ou non une paralysie*/
} Effet;

/*! \struct Attaque structures.h
 * \brief Definition de la structure Attaque.
 */
typedef struct Attaque{
    char id[TAILLE_ID];
    char nom[TAILLE_NOMS]; /*!< Nom de l'attaque de taille TAILLE_NOMS*/
    unsigned short int mana;
    unsigned short int portee; /*!< Portee en nombre de cases de l'attaque*/
    Effet effets[5]; /*!< Tableau des effets appliques par l'attaque, cela comprend les dgts de duree*/
} Attaque;

/*! \struct Classe structures.h
 * \brief Definition de la structure Classe.
 */
typedef struct Classe{
    char nom[25]; /*!< Nom de la classe de taille TAILLE_NOMS*/
    Attaque attaques[5]; /*!< Les attaques disponibles pour la classe*/
    unsigned short int points_deplacement_max; /*!< Les PD maximums accords a la classe*/
    unsigned short int PV_max; /*!< Les PV maximums accords a la classe*/
    unsigned short int mana_max; /*!< Le mana maximum accord a la classe*/
    Skins_Charac skins;
}Classe;

/*! \struct Personnage structures.h
 * \brief Definition de la structure Personnage.
 */
typedef struct Personnage{
    char nomPersonnage[TAILLE_NOMS];/*!< Nom du personnage de taille TAILLE_NOMS*/
    Classe* classe; /*!< La classe dont depend le personnage */
    unsigned short int points_deplacement; /*!< Les PD actuels du personnage*/
    unsigned short int PV; /*!< Les PV actuels du personnage*/
    unsigned short int mana; /*!< Le mana actuel du personnage*/
    Case *position; /*!< La case representant la position actuelle du personnage*/
    Effet effets[]; /*!< Un tableau d'effets representant les effets affectant le personnage*/
} Personnage;

/*! \struct Competence structures.h
 * \brief Definition de la structure competence.
 */
typedef struct Competence{
    char nomComp[TAILLE_NOMS]; /*!< Nom de la competence de taille TAILLE_NOMS*/
    char id[TAILLE_ID];   /*!< Un entier qui represente la competence*/
    unsigned int prix_competence; /*!< Points a depenser pour acquerir la competence */
    struct Competence *frere; /*!< Competence suivante sur le meme niveau */
    struct Competence *enfant; /*!< Premier enfant de la competence */
    struct Effet_Competence *effet;  /*!< Pointeur vers les effets de la competence */
    boolean connu; /*!< La competence est connue ou non */
}Competence;


/*! \struct Effet_competence structures.h
 * \brief Definition de la structure effet_competence
 */
typedef struct Effet_Competence{
    unsigned short int id; /*!< Un entier qui represente la competence*/
    char nom[TAILLE_NOMS]; /*!< Nom de la competence de taille TAILLE_NOMS*/
/* ........ */
} Effet_Competence;





/*! \struct Joueur structures.h
 * \brief Definition de la structure Joueur
 *
 * Un Joueur est compose principalement d'un nom et d'un tableau de Personnages.
 * Il reste a ajouter des attributs tels que l'arbre de competence.
 */
 typedef struct Joueur{
    char nomJoueur[TAILLE_NOMS]; /*!< Le nom identifiant le Joueur*/
    Personnage * groupe[TAILLE_MAX_GROUPE];/*!< Les personnages du joueur qui prennent part  la partie.*/
 } Joueur;

 /*! \struct NodeJoueur structures.h
 * \brief Definition de la structure NodeJoueur
 *
 * NodeJoueur est un noeud definit pour gerer une liste de Joueur
 */
 typedef struct NodeJoueur{
    Joueur * j; /*!< valeur du noeud*/
    struct NodeJoueur * next;/*!< Pointeur vers le noeud suivant*/
 } NodeJoueur;

/*! \struct ListJoueur structures.h
 * \brief Definition de la structure ListJoueur
 *
 * ListJoueur est une liste de noeuds pointant vers des Joueur
 */
 typedef struct ListJoueur{
    NodeJoueur * firstNodeJoueur; /*!<Premier noeud*/
    NodeJoueur * currentNodeJoueur; /*!<Noeud courant*/
    NodeJoueur * lastNodeJoueur; /*!<Dernier noeud*/
 }ListJoueur;





 /*! \struct Partie structures.h
 * \brief Definition de la structure Partie
 *
 * Contient les informations concernant la partie telles que la liste des joueurs participants
 * le nombre de tours. Cette structure permet de grer les tours
 * Peut eventuellement contenir l'information Carte si c'est utile.
 */
 typedef struct Partie{
    ListJoueur * participants; /*!< La liste de joueurs prennant parts  la partie*/
    int nbTours; /*!<Le nombre de tours actuel*/
    Carte * c; /*!<La Carte associee a la Partie*/
 }Partie;

#endif
